figure.figure-pair div.figure-pair-wrapper {
    display: flex;
    align-items: center;
    justify-content: center;
    align-content: center;
    gap: 1em;
}

figure.figure-pair div.figure-pair-left,
figure.figure-pair div.figure-pair-right {
    flex: 1; /* Ensure each image take half available space and gap is in middle */
    display: flex;
}

/* stick image to middle gap */
figure.figure-pair div.figure-pair-left {
    justify-content: flex-end;
}
figure.figure-pair div.figure-pair-right {
    justify-content: flex-start;
}
