"""
File        : mkdocs_table_of_figures/constants.py
Description : Theses are some constants from the package (same as setup.cfg)
Author      : Thibaud Briard - BRT, <thibaud.briard@outlook.com>
"""

from . import __version__

PACKAGE_NAME = 'mkdocs-table-of-figures'
VERSION = __version__

# Parse entrypoint identifier
MKDOCS_ENTRYPOINT = 'table-of-figures'
